@@ -17,8 +17,6 @@
 Command-line interface to the OpenStack Images API.
 """
 
-from __future__ import print_function
-
 import argparse
 import copy
 import getpass
@@ -31,18 +29,18 @@ import traceback
 
 from oslo_utils import encodeutils
 from oslo_utils import importutils
-import six.moves.urllib.parse as urlparse
+import urllib.parse
 
 import glanceclient
 from glanceclient._i18n import _
 from glanceclient.common import utils
 from glanceclient import exc
 
-from keystoneclient.auth.identity import v2 as v2_auth
-from keystoneclient.auth.identity import v3 as v3_auth
-from keystoneclient import discover
-from keystoneclient import exceptions as ks_exc
-from keystoneclient import session
+from keystoneauth1 import discover
+from keystoneauth1 import exceptions as ks_exc
+from keystoneauth1.identity import v2 as v2_auth
+from keystoneauth1.identity import v3 as v3_auth
+from keystoneauth1 import loading
 
 osprofiler_profiler = importutils.try_import("osprofiler.profiler")
 
@@ -51,34 +49,18 @@ SUPPORTED_VERSIONS = [1, 2]
 
 class OpenStackImagesShell(object):
 
-    def _append_global_identity_args(self, parser):
+    def _append_global_identity_args(self, parser, argv):
         # register common identity args
-        session.Session.register_cli_options(parser)
-        v3_auth.Password.register_argparse_arguments(parser)
+        parser.set_defaults(os_auth_url=utils.env('OS_AUTH_URL'))
 
-        parser.add_argument('--key-file',
-                            dest='os_key',
-                            help='DEPRECATED! Use --os-key.')
-
-        parser.add_argument('--ca-file',
-                            dest='os_cacert',
-                            help='DEPRECATED! Use --os-cacert.')
-
-        parser.add_argument('--cert-file',
-                            dest='os_cert',
-                            help='DEPRECATED! Use --os-cert.')
-
-        parser.add_argument('--os-tenant-id',
-                            default=utils.env('OS_TENANT_ID'),
-                            help='Defaults to env[OS_TENANT_ID].')
+        parser.set_defaults(os_project_name=utils.env(
+            'OS_PROJECT_NAME', 'OS_TENANT_NAME'))
+        parser.set_defaults(os_project_id=utils.env(
+            'OS_PROJECT_ID', 'OS_TENANT_ID'))
 
         parser.add_argument('--os_tenant_id',
                             help=argparse.SUPPRESS)
 
-        parser.add_argument('--os-tenant-name',
-                            default=utils.env('OS_TENANT_NAME'),
-                            help='Defaults to env[OS_TENANT_NAME].')
-
         parser.add_argument('--os_tenant_name',
                             help=argparse.SUPPRESS)
 
@@ -110,7 +92,19 @@ class OpenStackImagesShell(object):
         parser.add_argument('--os_endpoint_type',
                             help=argparse.SUPPRESS)
 
-    def get_base_parser(self):
+        loading.register_session_argparse_arguments(parser)
+        # Peek into argv to see if os-auth-token (or the deprecated
+        # os_auth_token) or the new os-token or the environment variable
+        # OS_AUTH_TOKEN were given. In which case, the token auth plugin is
+        # what the user wants. Else, we'll default to password.
+        default_auth_plugin = 'password'
+        token_opts = ['os-token', 'os-auth-token', 'os_auth-token']
+        if argv and any(i in token_opts for i in argv):
+            default_auth_plugin = 'token'
+        loading.register_auth_argparse_arguments(
+            parser, argv, default=default_auth_plugin)
+
+    def get_base_parser(self, argv):
         parser = argparse.ArgumentParser(
             prog='glance',
             description=__doc__.strip(),
@@ -146,14 +140,6 @@ class OpenStackImagesShell(object):
                                  'of schema that generates portions of the '
                                  'help text. Ignored with API version 1.')
 
-        parser.add_argument('--no-ssl-compression',
-                            dest='ssl_compression',
-                            default=True, action='store_false',
-                            help='DEPRECATED! This option is deprecated '
-                                 'and not used anymore. SSL compression '
-                                 'should be disabled by default by the '
-                                 'system SSL library.')
-
         parser.add_argument('-f', '--force',
                             dest='force',
                             default=False, action='store_true',
@@ -183,27 +169,30 @@ class OpenStackImagesShell(object):
         if osprofiler_profiler:
             parser.add_argument('--profile',
                                 metavar='HMAC_KEY',
+                                default=utils.env('OS_PROFILE'),
                                 help='HMAC key to use for encrypting context '
                                 'data for performance profiling of operation. '
                                 'This key should be the value of HMAC key '
                                 'configured in osprofiler middleware in '
-                                'glance, it is specified in paste '
+                                'glance, it is specified in glance '
                                 'configuration file at '
-                                '/etc/glance/api-paste.ini and '
-                                '/etc/glance/registry-paste.ini. Without key '
-                                'the profiling will not be triggered even '
-                                'if osprofiler is enabled on server side.')
+                                '/etc/glance/glance-api.conf and '
+                                '/etc/glance/glance-registry.conf. Without '
+                                'key the profiling will not be triggered even '
+                                'if osprofiler is enabled on server side. '
+                                'Defaults to env[OS_PROFILE].')
 
-        self._append_global_identity_args(parser)
+        self._append_global_identity_args(parser, argv)
 
         return parser
 
-    def get_subcommand_parser(self, version):
-        parser = self.get_base_parser()
+    def get_subcommand_parser(self, version, argv=None):
+        parser = self.get_base_parser(argv)
 
         self.subcommands = {}
         subparsers = parser.add_subparsers(metavar='<subcommand>')
-        submodule = utils.import_versioned_module(version, 'shell')
+        submodule = importutils.import_versioned_module('glanceclient',
+                                                        version, 'shell')
 
         self._find_actions(subparsers, submodule)
         self._find_actions(subparsers, self)
@@ -214,7 +203,8 @@ class OpenStackImagesShell(object):
 
     def _find_actions(self, subparsers, actions_module):
         for attr in (a for a in dir(actions_module) if a.startswith('do_')):
-            # I prefer to be hypen-separated instead of underscores.
+            # Replace underscores with hyphens in the commands
+            # displayed to the user
             command = attr[3:].replace('_', '-')
             callback = getattr(actions_module, attr)
             desc = callback.__doc__ or ''
@@ -260,13 +250,13 @@ class OpenStackImagesShell(object):
         v2_auth_url = None
         v3_auth_url = None
         try:
-            ks_discover = discover.Discover(session=session, auth_url=auth_url)
+            ks_discover = discover.Discover(session=session, url=auth_url)
             v2_auth_url = ks_discover.url_for('2.0')
             v3_auth_url = ks_discover.url_for('3.0')
         except ks_exc.ClientException as e:
             # Identity service may not support discover API version.
             # Lets trying to figure out the API version from the original URL.
-            url_parts = urlparse.urlparse(auth_url)
+            url_parts = urllib.parse.urlparse(auth_url)
             (scheme, netloc, path, params, query, fragment) = url_parts
             path = path.lower()
             if path.startswith('/v3'):
@@ -284,9 +274,7 @@ class OpenStackImagesShell(object):
 
         return (v2_auth_url, v3_auth_url)
 
-    def _get_keystone_session(self, **kwargs):
-        ks_session = session.Session.construct(kwargs)
-
+    def _get_keystone_auth_plugin(self, ks_session, **kwargs):
         # discover the supported keystone versions using the given auth url
         auth_url = kwargs.pop('auth_url', None)
         (v2_auth_url, v3_auth_url) = self._discover_auth_versions(
@@ -346,10 +334,9 @@ class OpenStackImagesShell(object):
                              "may not able to handle Keystone V3 credentials. "
                              "Please provide a correct Keystone V3 auth_url.")
 
-        ks_session.auth = auth
-        return ks_session
+        return auth
 
-    def _get_kwargs_for_create_session(self, args):
+    def _get_kwargs_to_create_auth_plugin(self, args):
         if not args.os_username:
             raise exc.CommandError(
                 _("You must provide a username via"
@@ -374,16 +361,11 @@ class OpenStackImagesShell(object):
                       "or prompted response"))
 
         # Validate password flow auth
-        project_info = (
-            args.os_tenant_name or args.os_tenant_id or (
-                args.os_project_name and (
-                    args.os_project_domain_name or
-                    args.os_project_domain_id
-                )
-            ) or args.os_project_id
-        )
-
-        if not project_info:
+        os_project_name = getattr(
+            args, 'os_project_name', getattr(args, 'os_tenant_name', None))
+        os_project_id = getattr(
+            args, 'os_project_id', getattr(args, 'os_tenant_id', None))
+        if not any([os_project_name, os_project_id]):
             # tenant is deprecated in Keystone v3. Use the latest
             # terminology instead.
             raise exc.CommandError(
@@ -416,10 +398,6 @@ class OpenStackImagesShell(object):
             'project_id': args.os_project_id,
             'project_domain_name': args.os_project_domain_name,
             'project_domain_id': args.os_project_domain_id,
-            'insecure': args.insecure,
-            'cacert': args.os_cacert,
-            'cert': args.os_cert,
-            'key': args.os_key
         }
         return kwargs
 
@@ -427,9 +405,7 @@ class OpenStackImagesShell(object):
         endpoint = self._get_image_url(args)
         auth_token = args.os_auth_token
 
-        auth_req = (hasattr(args, 'func') and
-                    utils.is_authentication_required(args.func))
-        if not auth_req or (endpoint and auth_token):
+        if endpoint and auth_token:
             kwargs = {
                 'token': auth_token,
                 'insecure': args.insecure,
@@ -437,11 +413,21 @@ class OpenStackImagesShell(object):
                 'cacert': args.os_cacert,
                 'cert': args.os_cert,
                 'key': args.os_key,
-                'ssl_compression': args.ssl_compression
             }
         else:
-            kwargs = self._get_kwargs_for_create_session(args)
-            kwargs = {'session': self._get_keystone_session(**kwargs)}
+            ks_session = loading.load_session_from_argparse_arguments(args)
+            auth_plugin_kwargs = self._get_kwargs_to_create_auth_plugin(args)
+            ks_session.auth = self._get_keystone_auth_plugin(
+                ks_session=ks_session, **auth_plugin_kwargs)
+            kwargs = {'session': ks_session}
+
+            if endpoint is None:
+                endpoint_type = args.os_endpoint_type or 'public'
+                service_type = args.os_service_type or 'image'
+                endpoint = ks_session.get_endpoint(
+                    service_type=service_type,
+                    interface=endpoint_type,
+                    region_name=args.os_region_name)
 
         return glanceclient.Client(api_version, endpoint, **kwargs)
 
@@ -457,7 +443,7 @@ class OpenStackImagesShell(object):
             except OSError as e:
                 # This avoids glanceclient to crash if it can't write to
                 # ~/.glanceclient, which may happen on some env (for me,
-                # it happens in Jenkins, as Glanceclient can't write to
+                # it happens in Jenkins, as glanceclient can't write to
                 # /var/lib/jenkins).
                 msg = '%s' % e
                 print(encodeutils.safe_decode(msg), file=sys.stderr)
@@ -487,27 +473,21 @@ class OpenStackImagesShell(object):
 
         def _get_subparser(api_version):
             try:
-                return self.get_subcommand_parser(api_version)
+                return self.get_subcommand_parser(api_version, argv)
             except ImportError as e:
-                if options.debug:
-                    traceback.print_exc()
                 if not str(e):
                     # Add a generic import error message if the raised
                     # ImportError has none.
                     raise ImportError('Unable to import module. Re-run '
                                       'with --debug for more info.')
                 raise
-            except Exception:
-                if options.debug:
-                    traceback.print_exc()
-                raise
 
         # Parse args once to find version
 
         # NOTE(flepied) Under Python3, parsed arguments are removed
         # from the list so make a copy for the first parsing
         base_argv = copy.deepcopy(argv)
-        parser = self.get_base_parser()
+        parser = self.get_base_parser(argv)
         (options, args) = parser.parse_known_args(base_argv)
 
         try:
@@ -519,7 +499,7 @@ class OpenStackImagesShell(object):
             endpoint = self._get_image_url(options)
             endpoint, url_version = utils.strip_version(endpoint)
         except ValueError:
-            # NOTE(flaper87): ValueError is raised if no endpoint is povided
+            # NOTE(flaper87): ValueError is raised if no endpoint is provided
             url_version = None
 
         # build available subcommands based on version
@@ -535,10 +515,11 @@ class OpenStackImagesShell(object):
         # Handle top-level --help/-h before attempting to parse
         # a command off the command line
         if options.help or not argv:
+            parser = _get_subparser(api_version)
             self.do_help(options, parser=parser)
             return 0
 
-        # Short-circuit and deal with help command right away.
+        # short-circuit and deal with help command right away.
         sub_parser = _get_subparser(api_version)
         args = sub_parser.parse_args(argv)
 
@@ -581,6 +562,12 @@ class OpenStackImagesShell(object):
         if not args.os_password and options.os_password:
             args.os_password = options.os_password
 
+        if args.debug:
+            # Set up the root logger to debug so that the submodules can
+            # print debug messages
+            logging.basicConfig(level=logging.DEBUG)
+            # for iso8601 < 0.1.11
+            logging.getLogger('iso8601').setLevel(logging.WARNING)
         LOG = logging.getLogger('glanceclient')
         LOG.addHandler(logging.StreamHandler())
         LOG.setLevel(logging.DEBUG if args.debug else logging.INFO)
@@ -595,12 +582,6 @@ class OpenStackImagesShell(object):
             args.func(client, args)
         except exc.Unauthorized:
             raise exc.CommandError("Invalid OpenStack Identity credentials.")
-        except Exception:
-            # NOTE(kragniz) Print any exceptions raised to stderr if the
-            # --debug flag is set
-            if args.debug:
-                traceback.print_exc()
-            raise
         finally:
             if profile:
                 trace_id = osprofiler_profiler.get().get_base_id()
@@ -671,4 +652,6 @@ def main():
     except KeyboardInterrupt:
         utils.exit('... terminating glance client', exit_code=130)
     except Exception as e:
+        if utils.debug_enabled(argv) is True:
+            traceback.print_exc()
         utils.exit(encodeutils.exception_to_unicode(e))
