@@ -13,11 +13,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import hashlib
 import json
 from oslo_utils import encodeutils
 from requests import codes
-import six
-from six.moves.urllib import parse
+import urllib.parse
 import warlock
 
 from glanceclient.common import utils
@@ -54,7 +54,7 @@ class Controller(object):
 
     @staticmethod
     def _wrap(value):
-        if isinstance(value, six.string_types):
+        if isinstance(value, str):
             return [value]
         return value
 
@@ -81,6 +81,7 @@ class Controller(object):
                 raise exc.HTTPBadRequest(msg)
         return sort
 
+    @utils.add_req_id_to_generator()
     def list(self, **kwargs):
         """Retrieve a listing of Image objects.
 
@@ -97,6 +98,7 @@ class Controller(object):
 
         def paginate(url, page_size, limit=None):
             next_url = url
+            req_id_hdr = {}
 
             while True:
                 if limit and page_size > limit:
@@ -105,7 +107,12 @@ class Controller(object):
                     next_url = next_url.replace("limit=%s" % page_size,
                                                 "limit=%s" % limit)
 
-                resp, body = self.http_client.get(next_url)
+                resp, body = self.http_client.get(next_url, headers=req_id_hdr)
+                # NOTE(rsjethani): Store curent request id so that it can be
+                # used in subsequent requests. Refer bug #1525259
+                req_id_hdr['x-openstack-request-id'] = \
+                    utils._extract_request_id(resp)
+
                 for image in body['images']:
                     # NOTE(bcwaldon): remove 'self' for now until we have
                     # an elegant way to pass it into the model constructor
@@ -114,11 +121,11 @@ class Controller(object):
                     # We do not validate the model when listing.
                     # This prevents side-effects of injecting invalid
                     # schema values via v1.
-                    yield self.unvalidated_model(**image)
+                    yield self.unvalidated_model(**image), resp
                     if limit:
                         limit -= 1
                         if limit <= 0:
-                            raise StopIteration
+                            return
 
                 try:
                     next_url = body['next']
@@ -134,19 +141,19 @@ class Controller(object):
         tags_url_params = []
 
         for tag in tags:
-            if not isinstance(tag, six.string_types):
+            if not isinstance(tag, str):
                 raise exc.HTTPBadRequest("Invalid tag value %s" % tag)
 
             tags_url_params.append({'tag': encodeutils.safe_encode(tag)})
 
-        for param, value in six.iteritems(filters):
-            if isinstance(value, six.string_types):
+        for param, value in filters.items():
+            if isinstance(value, str):
                 filters[param] = encodeutils.safe_encode(value)
 
-        url = '/v2/images?%s' % parse.urlencode(filters)
+        url = '/v2/images?%s' % urllib.parse.urlencode(filters)
 
         for param in tags_url_params:
-            url = '%s&%s' % (url, parse.urlencode(param))
+            url = '%s&%s' % (url, urllib.parse.urlencode(param))
 
         if 'sort' in kwargs:
             if 'sort_key' in kwargs or 'sort_dir' in kwargs:
@@ -170,60 +177,197 @@ class Controller(object):
             for dir in sort_dir:
                 url = '%s&sort_dir=%s' % (url, dir)
 
-        if isinstance(kwargs.get('marker'), six.string_types):
+        if isinstance(kwargs.get('marker'), str):
             url = '%s&marker=%s' % (url, kwargs['marker'])
 
-        for image in paginate(url, page_size, limit):
-            yield image
+        for image, resp in paginate(url, page_size, limit):
+            yield image, resp
 
-    def get(self, image_id):
+    @utils.add_req_id_to_object()
+    def _get(self, image_id, header=None):
         url = '/v2/images/%s' % image_id
-        resp, body = self.http_client.get(url)
+        header = header or {}
+        resp, body = self.http_client.get(url, headers=header)
         # NOTE(bcwaldon): remove 'self' for now until we have an elegant
         # way to pass it into the model constructor without conflict
         body.pop('self', None)
-        return self.unvalidated_model(**body)
+        return self.unvalidated_model(**body), resp
 
-    def data(self, image_id, do_checksum=True):
+    def get(self, image_id):
+        return self._get(image_id)
+
+    @utils.add_req_id_to_object()
+    def data(self, image_id, do_checksum=True, allow_md5_fallback=False):
         """Retrieve data of an image.
 
-        :param image_id:    ID of the image to download.
-        :param do_checksum: Enable/disable checksum validation.
-        :returns: An iterable body or None
+        When do_checksum is enabled, validation proceeds as follows:
+
+        1. if the image has a 'os_hash_value' property, the algorithm
+           specified in the image's 'os_hash_algo' property will be used
+           to validate against the 'os_hash_value' value.  If the
+           specified hash algorithm is not available AND allow_md5_fallback
+           is True, then continue to step #2
+        2. else if the image has a checksum property, MD5 is used to
+           validate against the 'checksum' value.  (If MD5 is not available
+           to the client, the download fails.)
+        3. else if the download response has a 'content-md5' header, MD5
+           is used to validate against the header value.  (If MD5 is not
+           available to the client, the download fails.)
+        4. if none of 1-3 obtain, the data is **not validated** (this is
+           compatible with legacy behavior)
+
+        :param image_id:    ID of the image to download
+        :param do_checksum: Enable/disable checksum validation
+        :param allow_md5_fallback:
+            Use the MD5 checksum for validation if the algorithm specified by
+            the image's 'os_hash_algo' property is not available
+        :returns: An iterable body or ``None``
         """
+        if do_checksum:
+            # doing this first to prevent race condition if image record
+            # is deleted during the image download
+            url = '/v2/images/%s' % image_id
+            resp, image_meta = self.http_client.get(url)
+            meta_checksum = image_meta.get('checksum', None)
+            meta_hash_value = image_meta.get('os_hash_value', None)
+            meta_hash_algo = image_meta.get('os_hash_algo', None)
+
         url = '/v2/images/%s/file' % image_id
         resp, body = self.http_client.get(url)
         if resp.status_code == codes.no_content:
-            return None
+            return None, resp
 
         checksum = resp.headers.get('content-md5', None)
         content_length = int(resp.headers.get('content-length', 0))
 
-        if do_checksum and checksum is not None:
-            body = utils.integrity_iter(body, checksum)
+        check_md5sum = do_checksum
+        if do_checksum and meta_hash_value is not None:
+            try:
+                hasher = hashlib.new(str(meta_hash_algo))
+                body = utils.serious_integrity_iter(body,
+                                                    hasher,
+                                                    meta_hash_value)
+                check_md5sum = False
+            except ValueError as ve:
+                if (str(ve).startswith('unsupported hash type') and
+                        allow_md5_fallback):
+                    check_md5sum = True
+                else:
+                    raise
 
-        return utils.IterableWithLength(body, content_length)
+        if do_checksum and check_md5sum:
+            if meta_checksum is not None:
+                body = utils.integrity_iter(body, meta_checksum)
+            elif checksum is not None:
+                body = utils.integrity_iter(body, checksum)
+            else:
+                # NOTE(rosmaita): this preserves legacy behavior to return the
+                # image data when checksumming is requested but there's no
+                # 'content-md5' header in the response.  Just want to make it
+                # clear that we're doing this on purpose.
+                pass
 
-    def upload(self, image_id, image_data, image_size=None):
+        return utils.IterableWithLength(body, content_length), resp
+
+    @utils.add_req_id_to_object()
+    def upload(self, image_id, image_data, image_size=None, u_url=None,
+               backend=None):
         """Upload the data for an image.
 
         :param image_id: ID of the image to upload data for.
         :param image_data: File-like object supplying the data to upload.
         :param image_size: Unused - present for backwards compatibility
+        :param u_url: Upload url to upload the data to.
+        :param backend: Backend store to upload image to.
         """
-        url = '/v2/images/%s/file' % image_id
+        url = u_url or '/v2/images/%s/file' % image_id
         hdrs = {'Content-Type': 'application/octet-stream'}
-        body = image_data
-        self.http_client.put(url, headers=hdrs, data=body)
+        if backend is not None:
+            hdrs['x-image-meta-store'] = backend
 
+        body = image_data
+        resp, body = self.http_client.put(url, headers=hdrs, data=body)
+        return (resp, body), resp
+
+    @utils.add_req_id_to_object()
+    def get_import_info(self):
+        """Get Import info from discovery endpoint."""
+        url = '/v2/info/import'
+        resp, body = self.http_client.get(url)
+        return body, resp
+
+    @utils.add_req_id_to_object()
+    def get_stores_info(self):
+        """Get available stores info from discovery endpoint."""
+        url = '/v2/info/stores'
+        resp, body = self.http_client.get(url)
+        return body, resp
+
+    @utils.add_req_id_to_object()
+    def delete_from_store(self, store_id, image_id):
+        """Delete image data from specific store."""
+        url = ('/v2/stores/%(store)s/%(image)s' % {'store': store_id,
+                                                   'image': image_id})
+        resp, body = self.http_client.delete(url)
+        return body, resp
+
+    @utils.add_req_id_to_object()
+    def stage(self, image_id, image_data, image_size=None):
+        """Upload the data to image staging.
+
+        :param image_id: ID of the image to upload data for.
+        :param image_data: File-like object supplying the data to upload.
+        :param image_size: Unused - present for backwards compatibility
+        """
+        url = '/v2/images/%s/stage' % image_id
+        resp, body = self.upload(image_id,
+                                 image_data,
+                                 u_url=url)
+        return body, resp
+
+    @utils.add_req_id_to_object()
+    def image_import(self, image_id, method='glance-direct', uri=None,
+                     backend=None, stores=None, allow_failure=True,
+                     all_stores=None):
+        """Import Image via method."""
+        headers = {}
+        url = '/v2/images/%s/import' % image_id
+        data = {'method': {'name': method}}
+        if stores:
+            data['stores'] = stores
+            if allow_failure:
+                data['all_stores_must_succeed'] = False
+        if backend is not None:
+            headers['x-image-meta-store'] = backend
+        if all_stores:
+            data['all_stores'] = True
+            if allow_failure:
+                data['all_stores_must_succeed'] = False
+
+        if uri:
+            if method == 'web-download':
+                data['method']['uri'] = uri
+            else:
+                raise exc.HTTPBadRequest('URI is only supported with method: '
+                                         '"web-download"')
+        resp, body = self.http_client.post(url, data=data, headers=headers)
+        return body, resp
+
+    @utils.add_req_id_to_object()
     def delete(self, image_id):
         """Delete an image."""
         url = '/v2/images/%s' % image_id
-        self.http_client.delete(url)
+        resp, body = self.http_client.delete(url)
+        return (resp, body), resp
 
+    @utils.add_req_id_to_object()
     def create(self, **kwargs):
         """Create an image."""
+        headers = {}
         url = '/v2/images'
+        backend = kwargs.pop('backend', None)
+        if backend is not None:
+            headers['x-image-meta-store'] = backend
 
         image = self.model()
         for (key, value) in kwargs.items():
@@ -232,21 +376,25 @@ class Controller(object):
             except warlock.InvalidOperation as e:
                 raise TypeError(encodeutils.exception_to_unicode(e))
 
-        resp, body = self.http_client.post(url, data=image)
+        resp, body = self.http_client.post(url, headers=headers, data=image)
         # NOTE(esheffield): remove 'self' for now until we have an elegant
         # way to pass it into the model constructor without conflict
         body.pop('self', None)
-        return self.model(**body)
+        return self.model(**body), resp
 
+    @utils.add_req_id_to_object()
     def deactivate(self, image_id):
         """Deactivate an image."""
         url = '/v2/images/%s/actions/deactivate' % image_id
-        return self.http_client.post(url)
+        resp, body = self.http_client.post(url)
+        return (resp, body), resp
 
+    @utils.add_req_id_to_object()
     def reactivate(self, image_id):
         """Reactivate an image."""
         url = '/v2/images/%s/actions/reactivate' % image_id
-        return self.http_client.post(url)
+        resp, body = self.http_client.post(url)
+        return (resp, body), resp
 
     def update(self, image_id, remove_props=None, **kwargs):
         """Update attributes of an image.
@@ -276,12 +424,16 @@ class Controller(object):
 
         url = '/v2/images/%s' % image_id
         hdrs = {'Content-Type': 'application/openstack-images-v2.1-json-patch'}
-        self.http_client.patch(url, headers=hdrs, data=image.patch)
+        resp, _ = self.http_client.patch(url, headers=hdrs, data=image.patch)
+        # Get request id from `patch` request so it can be passed to the
+        #  following `get` call
+        req_id_hdr = {
+            'x-openstack-request-id': utils._extract_request_id(resp)}
 
         # NOTE(bcwaldon): calling image.patch doesn't clear the changes, so
         # we need to fetch the image again to get a clean history. This is
         # an obvious optimization for warlock
-        return self.get(image_id)
+        return self._get(image_id, req_id_hdr)
 
     def _get_image_with_locations_or_fail(self, image_id):
         image = self.get(image_id)
@@ -290,12 +442,15 @@ class Controller(object):
                                      'API access to image locations')
         return image
 
+    @utils.add_req_id_to_object()
     def _send_image_update_request(self, image_id, patch_body):
         url = '/v2/images/%s' % image_id
         hdrs = {'Content-Type': 'application/openstack-images-v2.1-json-patch'}
-        self.http_client.patch(url, headers=hdrs, data=json.dumps(patch_body))
+        resp, body = self.http_client.patch(url, headers=hdrs,
+                                            data=json.dumps(patch_body))
+        return (resp, body), resp
 
-    def add_location(self, image_id, url, metadata):
+    def add_location(self, image_id, url, metadata, validation_data=None):
         """Add a new location entry to an image's list of locations.
 
         It is an error to add a URL that is already present in the list of
@@ -304,12 +459,18 @@ class Controller(object):
         :param image_id: ID of image to which the location is to be added.
         :param url: URL of the location to add.
         :param metadata: Metadata associated with the location.
+        :param validation_data: Validation data for the image.
         :returns: The updated image
         """
         add_patch = [{'op': 'add', 'path': '/locations/-',
                       'value': {'url': url, 'metadata': metadata}}]
-        self._send_image_update_request(image_id, add_patch)
-        return self.get(image_id)
+        if validation_data:
+            add_patch[0]['value']['validation_data'] = validation_data
+        response = self._send_image_update_request(image_id, add_patch)
+        # Get request id from the above update request and pass the same to
+        # following get request
+        req_id_hdr = {'x-openstack-request-id': response.request_ids[0]}
+        return self._get(image_id, req_id_hdr)
 
     def delete_locations(self, image_id, url_set):
         """Remove one or more location entries of an image.
@@ -332,7 +493,7 @@ class Controller(object):
         url_indices.sort(reverse=True)
         patches = [{'op': 'remove', 'path': '/locations/%s' % url_idx}
                    for url_idx in url_indices]
-        self._send_image_update_request(image_id, patches)
+        return self._send_image_update_request(image_id, patches)
 
     def update_location(self, image_id, url, metadata):
         """Update an existing location entry in an image's list of locations.
@@ -359,6 +520,9 @@ class Controller(object):
         patches = [{'op': 'replace',
                     'path': '/locations',
                     'value': list(url_map.values())}]
-        self._send_image_update_request(image_id, patches)
+        response = self._send_image_update_request(image_id, patches)
+        # Get request id from the above update request and pass the same to
+        # following get request
+        req_id_hdr = {'x-openstack-request-id': response.request_ids[0]}
 
-        return self.get(image_id)
+        return self._get(image_id, req_id_hdr)
